# ruff: noqa
# mypy: ignore-errors
jsonpath  # unused variable (src/seedcase_flower/section.py:78)
template_path  # unused variable (src/seedcase_flower/section.py:79)
jinja_variable  # unused variable (src/seedcase_flower/section.py:80)
mode  # unused variable (src/seedcase_flower/section.py:81)
_check_jsonpath  # unused method (src/seedcase_flower/section.py:83)
quarto_resource_tables  # unused variable (src/seedcase_flower/styles.py:9)
cls  # unused variable (src/seedcase_flower/section.py:85)
target  # unused variable (src/seedcase_flower/.venv/lib/python3.12/site-packages/_virtualenv.py:50)
handler  # unused variable (src/seedcase_flower/internals.py:20)
source  # unused variable (src/seedcase_flower/internals.py:20)
